#ifndef __OPENGL_OPENGL_RENDERER_H__
#define __OPENGL_OPENGL_RENDERER_H__

#include "../renderer.h"
#include "../vertex.h"
#include "glew.h"

class OpenGLRenderer : public Renderer {
	HDC dc;
	HGLRC glRC;

public:
	OpenGLRenderer(HDC dc) : dc(dc) {}
	virtual ~OpenGLRenderer() {}

	// 
	virtual bool initialize() {
		//  sNZtH[}bg
		PIXELFORMATDESCRIPTOR pfd =
		{
			sizeof(PIXELFORMATDESCRIPTOR),
			1,
			PFD_DRAW_TO_WINDOW | PFD_SUPPORT_OPENGL | PFD_DOUBLEBUFFER,		//Flags
			PFD_TYPE_RGBA,													//The kind of framebuffer. RGBA or palette.
			32,																//Colordepth of the framebuffer.
			0, 0, 0, 0, 0, 0,
			0,
			0,
			0,
			0, 0, 0, 0,
			24,																//Number of bits for the depthbuffer
			8,																//Number of bits for the stencilbuffer
			0,																//Number of Aux buffers in the framebuffer.
			PFD_MAIN_PLANE,
			0,
			0, 0, 0
		};
		int format = ChoosePixelFormat(dc, &pfd);
		if (format == 0)
			return false;	// YsNZtH[}bg

		// OpenGLgfoCXReLXgɎw̃sNZtH[}bgZbg
		if (!SetPixelFormat(dc, format, &pfd))
			return false;	// DCփtH[}bgݒ肷̂Ɏs

		// OpenGL context쐬
		glRC = wglCreateContext(dc);

		// 쐬ꂽReLXgJgiݎgp̃ReLXgjH
		if (!wglMakeCurrent(dc, glRC))
			return false;	// Ȃ݂c

		// Extention Functions[h
		glewExperimental = TRUE;
		GLenum err = glewInit();
		if (err != GLEW_OK)
			return false;	// ֐Q[hɎs

		return true;
	}

	// ㏈
	virtual void terminate() {
		//  JgReLXg𖳌ɂ
		wglMakeCurrent(NULL, NULL);

		// JgReLXg폜
		wglDeleteContext(glRC);
	}

	// `Jn
	virtual void begin() {
		wglMakeCurrent(dc, glRC);
		glClearColor(0.0f, 0.5f, 1.0f, 1.0f);
		glClear(GL_COLOR_BUFFER_BIT);
	}

	// _obt@`
	virtual void render(Vertex *vtx, unsigned vtxNum, unsigned primNum, const Float4x4 &matrix, const ScreenLayer &layer, TextureInstance &texture) {

		// oCgEw
		glPixelStorei(GL_UNPACK_ALIGNMENT, 4);

		// eNX`𗬂
		const TextureInstance::Desc &desc = texture.getDesc();
		const Texture::Desc texDesc = desc.texture->getDesc();
		unsigned mipmapLevel = 0;
		unsigned texWidth = texDesc.width;
		unsigned texHeight = texDesc.height;
		unsigned texBorder = 0;
		glTexImage2D(GL_TEXTURE_2D, mipmapLevel, GL_RGBA, texWidth, texHeight, texBorder, GL_RGBA, GL_UNSIGNED_BYTE, desc.texture->getResource());

		// ~bv}bvݒ
		unsigned magFilter = (desc.magFilter == TextureInstance::Nearest) ? GL_NEAREST : GL_NEAREST;
		unsigned minFilter = (desc.minFilter == TextureInstance::Nearest) ? GL_NEAREST : GL_NEAREST;
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, magFilter);	// g厞`
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, minFilter);	// k`

		// \t@̐ݒ
		unsigned op = (desc.op == TextureInstance::Decal) ? GL_DECAL : GL_MODULATE;
		glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, (GLfloat)op);

		// eNX`g܂錾
		glEnable(GL_TEXTURE_2D);

		// p
		glLoadIdentity();
		glMultMatrixf(layer.getProj().v);
		glMultMatrixf(layer.getView().v);
		glMultMatrixf(matrix.v);

		glBegin(GL_TRIANGLE_STRIP);
		for (unsigned i = 0; i < vtxNum; i++) {
			const Vertex &v = vtx[i];
			glColor4ub(v.r, v.g, v.b, v.a);
			glTexCoord2d(v.u, v.v);
			glVertex3f(v.x, v.y, v.z);
		}
		glEnd();
	}

	// Xbv
	virtual void swap() {
		glFlush();
		SwapBuffers(dc);
	}

	// `I
	virtual void end() {
		wglMakeCurrent(NULL, NULL);
	}
};

#endif
